<?php
namespace nur\sery\schema;

use ArrayAccess;
use LogicException;
use nulib\cl;
use nur\sery\schema\schemas\AssocSchema;
use nur\sery\schema\schemas\ListSchema;
use nur\sery\schema\schemas\ScalarSchema;

abstract class Schema implements ArrayAccess {
  /**
   * créer si besoin une nouvelle instance à partir d'une définition de schéma
   */
  static function ns(?Schema &$schema, $definition): self {
    if ($schema === null) {
      if (AssocSchema::isa_definition($definition)) {
        $schema = new AssocSchema($definition);
      } elseif (ListSchema::isa_definition($definition)) {
        $schema = new ListSchema($definition);
      } elseif (ScalarSchema::isa_definition($definition)) {
        $schema = new ScalarSchema($definition);
      } else {
        throw SchemaException::invalid_schema();
      }
    }
    return $schema;
  }

  /**
   * @var array définition du schéma, à redéfinir le cas échéant dans une classe
   * dérivée
   */
  const SCHEMA = null;

  /** @var array */
  protected $definition;

  /** retourner true si le schéma est de nature tableau associatif */
  function isAssoc(?AssocSchema &$assoc=null): bool { return false; }
  /** retourner true si le schéma est de nature liste */
  function isList(?ListSchema &$list=null): bool { return false; }
  /** retourner true si le schéma est de nature scalaire */
  function isScalar(?ScalarSchema &$scalar=null): bool { return false; }

  /**
   * Créer si besoin une nouvelle instance de {@link Value} qui référence la
   * variable $dest
   */
  abstract function nv(?Value &$value=null, &$dest=null, $key=null): Value;

  #############################################################################
  # key & properties

  function offsetExists($offset): bool {
    return array_key_exists($offset, $this->definition);
  }
  function offsetGet($offset) {
    if (!array_key_exists($offset, $this->definition)) return null;
    else return $this->definition[$offset];
  }
  function offsetSet($offset, $value): void {
    throw new LogicException("read-only");
  }
  function offsetUnset($offset): void {
    throw new LogicException("read-only");
  }

  const _PROPERTY_PKEYS = [];
  function __get($name) {
    $pkey = cl::get(static::_PROPERTY_PKEYS, $name, $name);
    return cl::pget($this->definition, $pkey);
  }
}
